#!/bin/bash

cdwriter=$1
shift; directory="$*"

if [ ! -d "$directory" ] || [ ! -b "$cdwriter" ]; then
	echo "usage: $0 cdwriter directory"
	echo "example: $0 /dev/sr0 /home/me/audio"
	exit
fi


#MP3 and OGG conversion
ls "$directory"/*.mp3 | while read audiofile; do
	if [ ! -f "$audiofile.wav" ]; then
		mpg321 --stereo --wav "$audiofile.wav" "$audiofile"
	fi
done

ls "$directory"/*.ogg | while read audiofile; do
	if [ ! -f "$audiofile.wav" ]; then
		oggdec -o "$audiofile.wav" "$audiofile"
	fi
done

audiosizes=""
totalsize=0
for audiosize in `ls -l "$directory"/*.wav | cut -f5 -d' '`; do
	let audiosize=audiosize/1048576
	audiosizes="$audiosizes$audiosize "
	let totalsize=totalsize+audiosize
done
let totalsize=totalsize*100/115 #totalsize=totalsize/1.15 (see "getaudiodirectorysize" function in "src/common-filesize.c")

#recheck if  CD media has enough space (new WAV files can be added by OGG/MP3 conversion)
udi=`hal-find-by-property --key block.device --string $cdwriter | grep volume`
cdmediasize=`hal-get-property --udi $udi --key volume.disc.capacity`
let cdmediasize=cdmediasize/1048576 #in MB

if (( $totalsize <= $cdmediasize ))
then cd "$directory"
	cdrecord -v gracetime=3 dev=$cdwriter -dao -audio -pad -useinfo -text "$directory"/*.wav | simpleburn-gauges $audiosizes
	cd - >/dev/null
fi
