typedef struct {
	gchar cddevice[20];
	gboolean hasmedia;
	gboolean hasrewritablemedia;
	gboolean hascdrommedia;
	gboolean hasdvdmedia;
	gboolean hasemptymedia;
	gboolean hasaudiocdmedia;
	gboolean hasdatamedia;
	guint64 mediasize;
	guint64 mediacapacity; //mediacapacity is reliable for empty CD and DVD, otherwise it is equal to mediasize
} T_cdinfos;


guint64 getdatamediasize ( gchar *device ) {
	gchar *command, *output;
	guint64 mediasize;
	gint exitvalue;
		
	command = g_strdup_printf( "isosize -d 1024 %s", device );
	g_spawn_command_line_sync( command, &output, NULL, &exitvalue, NULL );
	if (exitvalue == 0)
		mediasize = g_ascii_strtoull( output, NULL, 10);
	else
		mediasize = 0;
	g_free( command );
	g_free( output );
	
	return mediasize;
}


void setcdinfos ( gchar *device, T_cdinfos *cdinfos ) {
	gchar **udis;
	gchar *udi;
	gint udiscount;
	gint i;
	gchar *info;
	LibHalContext *halcontext;
	
	halcontext = libhal_ctx_new();
	libhal_ctx_set_dbus_connection (halcontext, dbus_bus_get (DBUS_BUS_SYSTEM, NULL));
	libhal_ctx_init (halcontext, NULL);
	
	cdinfos->hasmedia = FALSE;
	cdinfos->hasrewritablemedia = FALSE;
	cdinfos->hascdrommedia = FALSE;
	cdinfos->hasdvdmedia = FALSE;
	cdinfos->hasemptymedia = FALSE;
	cdinfos->hasaudiocdmedia = FALSE;
	cdinfos->hasdatamedia = FALSE;
	cdinfos->mediasize = 0;
	cdinfos->mediacapacity = 0;
	
	udis = libhal_manager_find_device_string_match (halcontext, "block.device", device, &udiscount, NULL);
	udi = NULL;
	for (i=0; i<udiscount; i++) { //there are 2 udis: one for the storage device and one for the volume (media), if present
		if (libhal_device_property_exists (halcontext, udis[i], "volume.is_disc", NULL)){
			udi = udis[i];
			break;
		}
	}
	
	if (udi != NULL) { //if there is a volume (media)
		info = libhal_device_get_property_string (halcontext, udi, "volume.disc.type", NULL);
		if ( info != NULL ) { //the media info are not loaded
			if (g_strncasecmp ("cd", info, 2) == 0) {
				cdinfos->hasmedia = TRUE;
				cdinfos->hascdrommedia = TRUE;
			} else {
				if (g_strncasecmp( "dvd", info, 3) == 0) {
					cdinfos->hasmedia = TRUE;
					cdinfos->hasdvdmedia = TRUE;
				}
				//HD-DVD and Blue-ray Disk are not supported yet
			}
			libhal_free_string (info);
			
			if ( cdinfos->hasmedia ) {
				cdinfos->hasrewritablemedia = libhal_device_get_property_bool (halcontext, udi, "volume.disc.is_rewritable", NULL);
				
				if ( cdinfos->hascdrommedia ) {
					cdinfos->hasemptymedia = libhal_device_get_property_bool (halcontext, udi, "volume.disc.is_blank", NULL);
					cdinfos->hasdatamedia = libhal_device_get_property_bool (halcontext, udi, "volume.disc.has_data", NULL);
					cdinfos->hasaudiocdmedia = libhal_device_get_property_bool (halcontext, udi, "volume.disc.has_audio", NULL);
				} else { //DVD
					info = libhal_device_get_property_string (halcontext, udi, "volume.fstype", NULL);
					if (g_strncasecmp ("iso9660", info, 7) == 0)
						cdinfos->hasdatamedia = TRUE;
					else
						cdinfos->hasemptymedia = TRUE;
					libhal_free_string (info);
				}
				
				cdinfos->mediacapacity = libhal_device_get_property_uint64 (halcontext, udi, "volume.disc.capacity", NULL) / 1024;
				if ( cdinfos->hasdatamedia )
					cdinfos->mediasize = getdatamediasize (device);
				else
					cdinfos->mediasize = cdinfos->mediacapacity;
			}
		}
	}
	
	libhal_ctx_shutdown (halcontext, NULL);
	libhal_ctx_free (halcontext);
}


void listcddevices (gchar ***devices, gboolean writersonly) {//'devices' is the address (for return value) of a null terminated array of strings
	gchar **udis;
	gint udiscount;
	gchar *devicename;
	gint i, j;
	LibHalContext *halcontext;

	halcontext = libhal_ctx_new();
	libhal_ctx_set_dbus_connection (halcontext, dbus_bus_get (DBUS_BUS_SYSTEM, NULL));
	libhal_ctx_init (halcontext, NULL);

	udis = libhal_manager_find_device_string_match (halcontext, "storage.drive_type", "cdrom", &udiscount, NULL);
	*devices = g_new (gchar *, udiscount+1);
	j=0;
	for (i=0; i<udiscount; i++) {
		devicename = libhal_device_get_property_string (halcontext, udis[i], "block.device", NULL);
		if (! writersonly || libhal_device_get_property_bool (halcontext, udis[i], "storage.cdrom.cdr", NULL)) {
			(*devices)[j] = devicename;
			j++;
		} else
			libhal_free_string (devicename);
	}
	(*devices)[j] = NULL;
	libhal_free_string_array (udis);

	libhal_ctx_shutdown (halcontext, NULL);
	libhal_ctx_free (halcontext);
}


void listcdwriters( gchar ***devices ) {
	listcddevices( devices, TRUE );
}


void listcdreaders( gchar ***devices ) {
	listcddevices( devices, FALSE );
}
